#include <InternalDisplay.hpp>
#include <lcd_elements.hpp>
#include <ClimateCard.hpp>
#include <DigitalInputCard.hpp>
#include <DigitalOutputCard.hpp>
#include <RemoteVariable.hpp>

struct cud_display_conf_t
{
    RemoteVariable *aqi;
    DigitalInputCard *inputCard;
    DigitalOutputCard *outputCard;
    ClimateCard *ac;
    HardwareSerial *uart;
    const int tx;
    const int rx;
    const uint32_t communication_baudrate;
    const uint32_t ota_baudrate;
    uint8_t light_pins[4];
    uint8_t fan_pins[3];
    uint8_t air_purifier_pin;
    uint8_t mosquito_zapper_pin;
};

class CUDDisplay : public ESPMegaDisplay
{
    public:
        CUDDisplay(cud_display_conf_t *conf);
        void begin();
        void display_init();
    private:
        // States Calculation
        bool get_lights_state();
        bool get_fans_state();
        // Callbacks
        void handle_input_change(uint8_t pin, bool state);
        void handle_output_change(uint8_t pin, bool state);
        void handle_aqi_change(char *value);
        void handle_payload(uint8_t type, uint8_t* payload, uint8_t length);
        // Change Display Elements
        void set_display_light_state(uint8_t row, bool state);
        void set_display_fan_state(uint8_t row, bool state);
        void set_display_mosquito_zapper_state(bool state);
        void set_display_air_purifier_state(bool state);
        // Local Variables
        cud_display_conf_t *conf;
};